'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _babelTypes = require('babel-types');

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _getAuxiliary = require('./get-auxiliary');

var _getAuxiliary2 = _interopRequireDefault(_getAuxiliary);

var _getComponentTemplate = require('./get-component-template');

var _getComponentTemplate2 = _interopRequireDefault(_getComponentTemplate);

var _getExports = require('./get-exports');

var _getExports2 = _interopRequireDefault(_getExports);

var _getImports2 = require('./get-imports');

var _getImports3 = _interopRequireDefault(_getImports2);

var _getLastPlainJsx = require('./get-last-plain-jsx');

var _getLastPlainJsx2 = _interopRequireDefault(_getLastPlainJsx);

var _injectGlobals = require('./inject-globals');

var _injectGlobals2 = _interopRequireDefault(_injectGlobals);

var _injectReactImport = require('./inject-react-import');

var _injectReactImport2 = _interopRequireDefault(_injectReactImport);

var _isStatelessCompatible = require('./is-stateless-compatible');

var _isStatelessCompatible2 = _interopRequireDefault(_isStatelessCompatible);

var _supportsStatelessComponents = require('./supports-stateless-components');

var _supportsStatelessComponents2 = _interopRequireDefault(_supportsStatelessComponents);

var _rewriteMemberExpressions = require('./rewrite-member-expressions');

var _rewriteMemberExpressions2 = _interopRequireDefault(_rewriteMemberExpressions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

/**
 * Create a React component definition from ast
 * Wraps a plain jsx template into an appropriate component if needed
 * @param  {Object} ast to wrap
 * @param  {string} name of the component
 * @return {Object} ast containing the wrapped component
 */
exports.default = function (ast, name) {
	let globals = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

	// Get the last JSX expression in the outermost scope
	const jsx = (0, _getLastPlainJsx2.default)(ast);

	// Get user-provided exports
	const exports = (0, _getExports2.default)(ast);

	// Check for default export
	const hasDefaultExport = exports.some(_babelTypes.isExportDefaultDeclaration);

	// Inject globals into ast
	(0, _injectGlobals2.default)(ast, globals);

	// If a default export OR plain jsx was found, assume we deal with
	// a complete JSX component definition
	if (hasDefaultExport || !jsx) {
		return ast;
	}

	// Check if the React version supports stateless components
	const stateless = (0, _supportsStatelessComponents2.default)(_react2.default) && (0, _isStatelessCompatible2.default)(ast);

	// Get user-provided imports

	var _getImports = (0, _getImports3.default)(ast);

	const imports = _getImports.imports;

	// Stuff we found so far

	const excludes = [].concat(_toConsumableArray(imports), [jsx], _toConsumableArray(exports));

	// Get remaining code
	const auxiliary = (0, _getAuxiliary2.default)(ast, excludes, stateless);

	// Push the created react component into ast
	(0, _babelTraverse2.default)(ast, {
		Program: {
			exit: function exit(path) {
				const NAME = path.scope.generateUidIdentifier(name);

				// Get a react component ast
				const component = (0, _getComponentTemplate2.default)({ stateless: stateless })({
					AUXILIARY: auxiliary.map(path => path.node),
					JSX: jsx.node,
					NAME: NAME
				});

				// Create a default export for it
				const defaultExport = (0, _babelTypes.exportDefaultDeclaration)(NAME);
				path.pushContainer('body', [component, defaultExport]);
			}
		}
	});

	// Rewrite member expressions for stateless components
	// to ease the transition for older projects
	// - this.props => props
	// - this.context => context
	if (stateless) {
		(0, _rewriteMemberExpressions2.default)(ast);
	}

	// Inject react import if it is missing
	(0, _injectReactImport2.default)(ast);

	// Remove jsx
	jsx.remove();

	// Remove auxiliary code
	auxiliary.map(aux => aux.remove());

	return ast;
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];