'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _commonTags = require('common-tags');

var _emphasize = require('emphasize');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const deprecation = _chalk2.default.yellow('[ ⚠  Deprecation ⚠ ]');
const reference = 'https://facebook.github.io/react/docs/context.html';
const removal = _chalk2.default.yellow(_commonTags.oneLine`Global configuration for transform
	"react" and will be removed with version "1.0.0".`);

function pretty(data) {
	let strip = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

	return JSON.stringify(data, null, '').replace(/"/g, strip ? '' : '\'');
}

/**
 * Log a deprecation warning about global configuration
 * @param  {Object} application to log on
 */

exports.default = (application, file, globals) => {
	const types = Object.entries(globals).reduce((registry, entry) => {
		var _entry = _slicedToArray(entry, 2);

		const propertyName = _entry[0];
		const propertyValue = _entry[1];

		return _extends({}, registry, {
			[propertyName]: `types.${ typeof propertyValue }`
		});
	}, {});

	const message = _chalk2.default.yellow(_commonTags.oneLine`${ deprecation } Found "global"
		configuration for ${ file.pattern.id }:${ file.name }.`);

	const warning = _commonTags.stripIndent`
	${ message } ${ removal }
	Replace the configuration with a <Root /> pattern providing React context.
	See ${ _chalk2.default.bold(reference) } for reference.

	  ${ (0, _emphasize.highlight)('js', `// Example definition
	  import React, {PropTypes as types} from 'react';

	  export default class Root extends React.Component {
	    static childContextTypes = {
	      return ${ pretty(types, true) }
	    };

	    getChildContext() {
	      return ${ pretty(globals) }
	    }
	  }

	  // Example usage
	  import React, {PropTypes as types} from 'react';

	  export default class Root extends React.Component {
	    static contextTypes = {
	      return ${ pretty(types, true) }
	    };

	    render() {
	      console.log(this.context); // => ${ pretty(globals) }
	      return <div />;
	    }
	  `).value }}
	`;

	application.log.warn(warning);
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];