'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @typedef Imports
 * @type Object
 * @property {Array}  an Array of the paths belonging to
						import statements / require calls
 * @property {Array}  an Array of the imported names
 */

/**
 * Get all require calls and import statements
 * @param  {Object}   ast to search in
 * @return {Imports}  The import paths and their names
 */
exports.default = ast => {
	const imports = [];
	const identifiers = [];

	(0, _babelTraverse2.default)(ast, {
		ImportDeclaration: function ImportDeclaration(path) {
			imports.push(path.node);
			identifiers.push(path.node.source.value);
		},
		CallExpression: function CallExpression(path) {
			if (path.node.callee.name === 'require') {
				if (path.parent.type === 'VariableDeclarator') {
					imports.push(path.parentPath.parent);
				} else {
					imports.push(path.parent);
				}
				identifiers.push(path.node.arguments[0]);
			}
		}
	});

	return { imports: imports, identifiers: identifiers };
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];