'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _bluebird = require('bluebird');

let convertCode = (() => {
	var _ref = (0, _bluebird.coroutine)(function* (application, file, settings) {
		const deprecationMapping = {
			default: function _default(item) {
				const lines = file.source.toString().split('\n').slice(Math.max(item.line - 2, 0), 4);

				const message = [`[ ⚠  Deprecation ⚠ ]`, `${ item.type } "${ item.key }" is deprecated.`, `Use ${ item.alternative } instead.`, `Found in`, `${ file.pattern.id }/${ file.name }:${ item.line }:${ item.column }`].join(' ');

				const snippet = [`\n\n`, `${ _chalk2.default.grey(item.line - 1) } ${ lines[0] }\n`, `${ _chalk2.default.yellow(item.line) } ${ lines[1] }\n`, `${ _chalk2.default.grey(item.line + 1) } ${ lines[2] }\n`].join('');

				application.log.warn(_chalk2.default.yellow(message), snippet);
			}
		};

		const options = settings.opts || { globals: {} };
		const parseKey = ['react', 'parse', file.path, JSON.stringify(options.globals)].join(':');

		const transformKey = ['react', 'transform', file.path, JSON.stringify(options.globals)].join(':');

		const mtime = file.mtime || file.fs.node.mtime;

		const ast = application.cache.get(parseKey, mtime) || (0, _parse2.default)(file.buffer.toString('utf-8'));

		application.cache.set(parseKey, mtime, ast);

		// manifest.name is used as name for wrapped components
		const manifestName = file.pattern.manifest.name;

		// if (Object.keys(options.globals).length > 0) {
		// 	deprecateGlobalConfiguration(application, file, options.globals || {});
		// }

		// Get the component ast
		const component = (0, _createReactComponent2.default)(ast, (0, _pascalCase2.default)(manifestName), options.globals);

		(ast.deprecations || []).forEach(function (deprecation) {
			const fn = deprecationMapping[deprecation] || deprecationMapping.default;
			fn(deprecation);
		});

		const depNames = Object.keys(file.dependencies).map(function (name) {
			return (0, _pascalCase2.default)(name);
		});

		// Search for implicit dependencies
		const implicitDependencyRegistry = (0, _getImplicitDependencies2.default)(ast, depNames);
		const implicitDependencies = (0, _lodash.values)(implicitDependencyRegistry);

		// Implicit dependencies are deprecated, warn users about them
		if (implicitDependencies.length > 0) {
			(0, _deprecateImplicitDependencies2.default)(application, file, implicitDependencyRegistry);
			// Inject them anyway
			(0, _injectImplicitDependencies2.default)(component, implicitDependencyRegistry);
		}

		// Check if dependencies are found in pattern.dependencies,
		// get array of required dependency names
		const dependencyNames = yield (0, _getResolvableDependencies2.default)(component, file);

		if (settings.convertDependencies || settings.resolveDependencies) {
			// convert squashed dependencies
			yield Promise.all(dependencyNames.map((() => {
				var _ref2 = (0, _bluebird.coroutine)(function* (name) {
					const dependencies = file.dependencies;

					const dependency = dependencies[name];
					if (typeof dependency === 'undefined') {
						return Promise.resolve();
					}
					dependencies[name] = yield convertCode(application, dependency, settings);
					return dependencies[name];
				});

				return function (_x4) {
					return _ref2.apply(this, arguments);
				};
			})()));
		}

		const babelOptions = (0, _lodash.merge)({}, (0, _lodash.omit)(options, ['globals']), babelDefaults);

		// TODO: enable this when switching to babel 6
		// some babel-presets are required
		/* requiredBabelPresets.forEach(requiredBabelPreset => {
  	if (babelOptions.presets.includes(requiredBabelPreset) === false) {
  		babelOptions.presets.push(requiredBabelPreset);
  	}
  }); */

		// TODO: switch to babel6, use transformFromAst
		// TODO: transform should move to babel transform completely

		var _ref3 = application.cache.get(transformKey, mtime) || (0, _babelCore.transform)((0, _babelGenerator2.default)(component.program).code, babelOptions);

		const code = _ref3.code;


		application.cache.set(transformKey, mtime, { code: code });

		file.buffer = code;
		file.meta.react = (0, _lodash.merge)({}, file.meta.react, {
			ast: component,
			dependencyNames: dependencyNames
		});
		return file;
	});

	return function convertCode(_x, _x2, _x3) {
		return _ref.apply(this, arguments);
	};
})();

var _types = require('./types');

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _babelCore = require('babel-core');

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _lodash = require('lodash');

var _pascalCase = require('pascal-case');

var _pascalCase2 = _interopRequireDefault(_pascalCase);

var _createReactComponent = require('./create-react-component');

var _createReactComponent2 = _interopRequireDefault(_createReactComponent);

var _deprecateImplicitDependencies = require('./deprecate-implicit-dependencies');

var _deprecateImplicitDependencies2 = _interopRequireDefault(_deprecateImplicitDependencies);

var _getImplicitDependencies = require('./get-implicit-dependencies');

var _getImplicitDependencies2 = _interopRequireDefault(_getImplicitDependencies);

var _getResolvableDependencies = require('./get-resolvable-dependencies');

var _getResolvableDependencies2 = _interopRequireDefault(_getResolvableDependencies);

var _injectImplicitDependencies = require('./inject-implicit-dependencies');

var _injectImplicitDependencies2 = _interopRequireDefault(_injectImplicitDependencies);

var _parse = require('./parse');

var _parse2 = _interopRequireDefault(_parse);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// TODO: enable this when switching to babel 6
/* const babelDefaults = {
	presets: []
};

const requiredBabelPresets = ['react', 'es2015']; */

const babelDefaults = {};
// import deprecateGlobalConfiguration from './deprecate-global-configuration';


function getReactTransformFunction(application, config) {
	return (() => {
		var _ref4 = (0, _bluebird.coroutine)(function* (file, _, configuration) {
			const settings = (0, _lodash.merge)({}, config, configuration);
			return yield convertCode(application, file, settings);
		});

		return function (_x5, _x6, _x7) {
			return _ref4.apply(this, arguments);
		};
	})();
}

exports.default = application => {
	const config = application.configuration.transforms.react;

	return getReactTransformFunction(application, config);
};

/* flow-include var module: { change_code?: number }; module = {}; */


module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];