'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _babelTypes = require('babel-types');

var _injectionTemplate = require('./injection-template');

var _injectionTemplate2 = _interopRequireDefault(_injectionTemplate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

/**
 * Inject global variables into a react component,
 * making them available via this[globalKey] (if possible, deprecated)
 * and on node's global object
 * @param  {Object} ast        generated by babel
 * @param  {Object} globals    global variables to inject
 * @return {Object}            ast with injected globals
 */
exports.default = function (ast) {
	let globals = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

	ast.deprecations = [];

	if (Object.keys(globals).length === 0) {
		return ast;
	}

	// This is some seriously twisted legacy burden
	// Until recently we injected globals into the scope of react components -
	// to do this in interop with all react component cases we have to
	// - find references to this.<globalKey>
	// - replace it with global.<globalKey>
	// - deprecate this stuff!
	(0, _babelTraverse2.default)(ast, {
		MemberExpression: function MemberExpression(path) {
			const isGlobalLegacyReference = Object.keys(globals).some(key => path.matchesPattern(`this.${ key }`));
			if (isGlobalLegacyReference) {
				const previous = (0, _babelGenerator2.default)(path.node).code.split('.').slice(1);
				const replaced = ['global'].concat(_toConsumableArray(previous)).join('.');

				const pos = path.node.loc.start;
				ast.deprecations.push({
					key: previous[0],
					type: 'globalContextAccess',
					line: pos.line,
					column: pos.column,
					alternative: replaced
				});
				path.replaceWith((0, _babelTypes.identifier)(replaced));
			}
		}
	});

	if (ast.hasGlobals) {
		return ast;
	}

	ast.hasGlobals = true;

	// Inject access to global config
	// as global.<key>
	(0, _babelTraverse2.default)(ast, {
		Program: {
			exit: function exit(path) {
				const IDENTIFIER = path.scope.generateUidIdentifier('injection');

				const INJECTION = (0, _babelTemplate2.default)(`var ${ IDENTIFIER.name } = ${ JSON.stringify(globals) };`)();

				const injection = (0, _injectionTemplate2.default)({
					IDENTIFIER: IDENTIFIER,
					INJECTION: INJECTION
				});

				path.unshiftContainer('body', injection);
			}
		}
	});

	return ast;
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];