'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _babelCodeFrame = require('babel-code-frame');

var _babelCodeFrame2 = _interopRequireDefault(_babelCodeFrame);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _babylon = require('babylon');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const settings = {
	allowImportExportEverywhere: true,
	allowReturnOutsideFunction: true,
	sourceType: 'module',
	plugins: ['jsx', 'asyncFunctions', 'classConstructorCall', 'doExpressions', 'trailingFunctionCommas', 'objectRestSpread', 'decorators', 'classProperties', 'exportExtensions', 'exponentiationOperator', 'asyncGenerators', 'functionBind', 'functionSent']
};

function partialParse(source, _ref) {
	let line = _ref.line;

	if (source.length === 0) {
		return null;
	}

	const partial = source.split('\n').slice(0, line - 1).join('\n');

	try {
		return (0, _babylon.parse)(partial, settings);
	} catch (error) {
		partialParse(source, error.loc);
	}
}

function getLastStatement(ast) {
	const statements = [];

	(0, _babelTraverse2.default)(ast, {
		Statement: {
			exit: function exit(_ref2) {
				let node = _ref2.node;

				statements.push(node);
			}
		}
	});

	return statements[statements.length - 1];
}

function getAsiError(error, source) {
	if (typeof error.loc === 'undefined') {
		return error;
	}

	const partialAst = partialParse(source, error.loc);

	if (partialAst === null) {
		return error;
	}

	const lastStatement = getLastStatement(partialAst);

	if (typeof lastStatement === 'undefined') {
		return error;
	}

	error.asi = true;
	error.loc = lastStatement.loc.end;
	error.loc.column += 1;
	error.message = [`Detected possible missing semi-colon before plain jsx block:`, ``, error.message].join('\n');

	return error;
}

exports.default = source => {
	try {
		return (0, _babylon.parse)(source, settings);
	} catch (error) {
		const augmentedError = getAsiError(error, source);

		if (augmentedError.loc) {
			var _error$loc = error.loc;
			const line = _error$loc.line;
			const column = _error$loc.column;

			const lines = source.split('\n');

			error.message = [error.message, '', (0, _babelCodeFrame2.default)(source, line, column, {
				highlightCode: true
			}), '', augmentedError.asi && [`You can fix this by adding a semi-colon at line ${ line }:`, `${ lines[line - 1] };`, ``, `or wrapping jsx in parenthesis, e.g.:`, `(<div />)`, ``].join('\n')].join('\n');
		}
		throw error;
	}
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];