'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _babelTraverse = require('babel-traverse');

var _babelTraverse2 = _interopRequireDefault(_babelTraverse);

var _babelTypes = require('babel-types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Get the top level scope for a given path
 * @param  {Object} path
 * @return {Object} path holding the top-level scope
 */
function getTopScopePath(path) {
	while (path.scope.parent && !(0, _babelTypes.isScopable)(path)) {
		path = path.scope.parent.path;
	}
	return path;
}

/**
 * Check if a given path has it scope in the top level
 * @param  {Object}  path
 * @return {Boolean}
 */
function hasRenderParent(path) {
	const scopePath = getTopScopePath(path);

	if ((0, _babelTypes.isProgram)(scopePath)) {
		return true;
	}

	return false;
}

/**
 * Check if a node is a react-specific member expression
 * @param  {Object}  node
 * @return {Boolean}
 */
function isReactMemberExpression(node) {
	if (node.object && !(0, _babelTypes.isThisExpression)(node.object) && !(0, _babelTypes.isMemberExpression)(node.object)) {
		return false;
	}

	if ((0, _babelTypes.isMemberExpression)(node.object)) {
		return isReactMemberExpression(node.object);
	}

	return ['props', 'context'].includes(node.property.name);
}

/**
 * Rewrite member expressions matching this.props or this.context
 * @TODO: Rewrite member expression in lower scopes if applicable
 * - arrow functions
 * - blocks
 */

exports.default = ast => {
	(0, _babelTraverse2.default)(ast, {
		MemberExpression: {
			enter: function enter(path) {
				if (!isReactMemberExpression(path.node)) {
					return;
				}

				if (!hasRenderParent(path)) {
					path.skip();
					return;
				}

				const sliced = (0, _babelGenerator2.default)(path.node).code.split('.').slice(1).join('.');

				path.replaceWith((0, _babelTypes.identifier)(sliced));
			}
		}
	});

	return ast;
};

module.change_code = 1; // eslint-disable-line

module.exports = exports['default'];